@@ -1,7 +1,7 @@
-/* $Id: pkgconfig.c,v 1.30.6.1 2005/04/25 20:30:26 mipsator Exp $ */
+/* $Id: pkgconfig.c 1971 2007-04-15 14:13:18Z coudercd $ */
 
 /*
- * Copyright (c) 2003-2004 Damien Couderc
+ * Copyright (c) 2003-2005 Damien Couderc
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,13 +38,15 @@
 
 #include <dirent.h>
 #include <errno.h>
-#include <stdio.h>
 #include <stdlib.h>
 
+#include "compat/pmk_ctype.h"
+#include "compat/pmk_stdio.h"
 #include "compat/pmk_stdbool.h"
 #include "compat/pmk_string.h"
 #include "common.h"
 #include "hash.h"
+#include "hash_tools.h"
 #include "parse.h"
 #include "pkgconfig.h"
 #include "premake.h"
@@ -57,6 +59,10 @@
  * env PKG_CONFIG_LIBDIR => replace default location (aka prefix/lib/pkgconfig)
  */
 
+/**********
+ constants
+************************************************************************/
+
 #define PKGCFG_HTABLE_SIZE	32
 #define PKGCFG_TOK_ADDCT	1
 
@@ -68,6 +74,10 @@
 #define PKGCFG_KW_CFLGS	5
 #define PKGCFG_KW_CFLCT	6
 
+/**********
+ variables
+************************************************************************/
+
 /* pc file keywords */
 pkgkw	kw_pkg[] = {
 		{"Name",	PKGCFG_KW_NAME},
@@ -82,9 +92,22 @@ pkgkw	kw_pkg[] = {
 size_t	nb_pkgkw = sizeof(kw_pkg) / sizeof(pkgkw);
 
 
-/*
+/**********
+ functions
+************************************************************************/
+
+/***************
+ pkgcell_init()
+
+ DESCR
 	initialize pkgcell structure
-*/
+
+ IN
+	NONE
+
+ OUT
+	pkgcell structure or NULL
+************************************************************************/
 
 pkgcell *pkgcell_init(void) {
 	pkgcell *ppc;
@@ -109,9 +132,19 @@ pkgcell *pkgcell_init(void) {
 	return(ppc);
 }
 
-/*
+
+/******************
+ pkgcell_destroy()
+
+ DESCR
 	free pkgcell structure
-*/
+
+ IN
+	ppc : pkgcell structure
+
+ OUT
+	NONE
+************************************************************************/
 
 void pkgcell_destroy(pkgcell *ppc) {
 	if (ppc->name != NULL)
@@ -127,17 +160,25 @@ void pkgcell_destroy(pkgcell *ppc) {
 		free(ppc->requires);
 
 	da_destroy(ppc->cflags);
-
 	da_destroy(ppc->libs);
-
 	hash_destroy(ppc->variables);
 
 	free(ppc);
 }
 
-/*
+
+/***************
+ pkgdata_init()
+
+ DESCR
 	initialize pkgdata structure
-*/
+
+ IN
+	NONE
+
+ OUT
+	pkgdata structure or NULL
+************************************************************************/
 
 pkgdata *pkgdata_init(void) {
 	pkgdata	*ppd;
@@ -174,48 +215,85 @@ pkgdata *pkgdata_init(void) {
 	return(ppd);
 }
 
-/*
-	free pkgcell structure
-*/
+
+/******************
+ pkgdata_destroy()
+
+ DESCR
+	free pkgdata structure
+
+ IN
+	ppd : pkgdata structure
+
+ OUT
+	NONE
+************************************************************************/
 
 void pkgdata_destroy(pkgdata *ppd) {
 #ifdef PKGCFG_DEBUG
-debugf("clean pc files hash");
+	debugf("clean pc files hash");
 #endif
 	hash_destroy(ppd->files);
 
 #ifdef PKGCFG_DEBUG
-debugf("clean cells hash");
+	debugf("clean cells hash");
 #endif
 	hash_destroy(ppd->cells);
 
 #ifdef PKGCFG_DEBUG
-debugf("clean modules list");
+	debugf("clean modules list");
 #endif
 	da_destroy(ppd->mods);
 
 #ifdef PKGCFG_DEBUG
-debugf("clean structure");
+	debugf("clean structure");
 #endif
 	free(ppd);
 }
 
-/*
+
+/*************
+ skip_blank()
+
+ DESCR
+	skip blank character(s)
+
+ IN
+	pstr: current parsing cursor
+
+ OUT
+	new parsing cursor
+***********************************************************************/
+
+char *skip_blank(char *pstr) {
+	while (isblank(*pstr) != 0) {
+		pstr++;
+	}
+	return(pstr);
+}
+
+
+/***********
+ scan_dir()
+
+ DESCR
 	scan given directory for pc files
 
+ IN
 	dir : directory to scan
 	ppd : pkgdata structure
 
-	return : boolean
-*/
+ OUT
+	boolean
+************************************************************************/
 
 bool scan_dir(char *dir, pkgdata *ppd) {
 	struct dirent	*pde;
-	DIR		*pdir;
-	char		*pstr,
-			 buf[MAXPATHLEN],
-			 fpath[MAXPATHLEN];
-	size_t		 l;
+	DIR				*pdir;
+	char			*pstr,
+					 buf[MAXPATHLEN],
+					 fpath[MAXPATHLEN];
+	size_t			 l;
 
 	/* open directory */
 	pdir = opendir(dir);
@@ -248,8 +326,7 @@ bool scan_dir(char *dir, pkgdata *ppd) {
 				/* build full path of pc file */
 				strlcpy(fpath, dir, sizeof(fpath)); /* no check */
 				strlcat(fpath, "/", sizeof(fpath)); /* no check */
-				if (strlcat_b(fpath, pstr,
-						sizeof(fpath)) == false) {
+				if (strlcat_b(fpath, pstr, sizeof(fpath)) == false) {
 					/* XXX err msg ? */
 					closedir(pdir);
 					return(false);
@@ -259,7 +336,7 @@ bool scan_dir(char *dir, pkgdata *ppd) {
 				/* XXX detect if the package has already been detected ? */
 
 #ifdef PKGCFG_DEBUG
-debugf("add module '%s' with file '%s'", buf, pstr);
+				debugf("add module '%s' with file '%s'", buf, pstr);
 #endif
 			}
 		}
@@ -270,60 +347,82 @@ debugf("add module '%s' with file '%s'", buf, pstr);
 	return(true);
 }
 
-/*
+
+/**************
+ pkg_collect()
+
+ DESCR
 	collect packages data
 
+ IN
 	pkglibdir : default packages data directory
 	ppd : pkgdata structure
 
+ OUT
 	return : boolean
-*/
+************************************************************************/
 
 bool pkg_collect(char *pkglibdir, pkgdata *ppd) {
-	char		*pstr;
-	dynary		*pda;
+	char			*pstr;
+	dynary			*ppath,
+					*ppcpath;
 	unsigned int	 i;
 
 	pstr = getenv(PMKCFG_ENV_PATH);
 	if (pstr != NULL) {
 		/* also scan path provided in env variable */
-		pda = str_to_dynary(pstr, PKGCFG_CHAR_PATH_SEP);
-		
-		for (i = 0 ; i < da_usize(pda) ; i++) {
-			pstr = da_idx(pda, i);
+		ppath = str_to_dynary(pstr, PKGCFG_CHAR_PATH_SEP);
+
+		for (i = 0 ; i < da_usize(ppath) ; i++) {
+			pstr = da_idx(ppath, i);
 			if (scan_dir(pstr, ppd) == false) {
-				da_destroy(pda);
+				da_destroy(ppath);
 				return(false);
 			}
 		}
 
-		da_destroy(pda);
+		da_destroy(ppath);
 	}
 
+	/* check if pkg-config library directory has been set in env variable */
 	pstr = getenv(PMKCFG_ENV_LIBDIR);
 	if (pstr == NULL) {
-		/* scan default pkgconfig lib directory */
-		if (scan_dir(pkglibdir, ppd) == false)
-			return(false);
+		/* env variable is not set, getting pmk.conf variable */
+		ppcpath = str_to_dynary(pkglibdir, PKGCFG_CHAR_PATH_SEP);
 	} else {
-		/* scan overrided pkgconfig libdir */
-		if (scan_dir(pstr, ppd) == false)
-			return(false);
+		/* else use env variable */
+		ppcpath = str_to_dynary(pstr, PKGCFG_CHAR_PATH_SEP);
 	}
-	
+
+	/* scan all path for pc files */
+	for (i = 0 ; i < da_usize(ppcpath) ; i++) {
+		/* get directory */
+		pstr = da_idx(ppcpath, i);
+
+		/* scan directory */
+		if (scan_dir(pstr, ppd) == false) {
+			return(false);
+		}
+	}
+
 	return(true);
 }
 
 
-/*
+/****************
+ parse_keyword()
+
+ DESCR
 	parse keyword and set data
 
+ IN
 	ppc : pkgcell structure
 	kword : keyword
 	value : value to assign
 
-	return : boolean
-*/
+ OUT
+	boolean
+************************************************************************/
 
 bool parse_keyword(pkgcell *ppc, char *kword, char *value) {
 	unsigned int	i;
@@ -389,21 +488,27 @@ bool parse_keyword(pkgcell *ppc, char *kword, char *value) {
 	return(true);
 }
 
-/*
+
+/********************
+ process_variables()
+
+ DESCR
 	process string to substitute variables with their values
 
+ IN
 	pstr : string to process
 	pht : hash table where variables are stored
 
-	return : new string or NULL
-*/
+ OUT
+	new string or NULL
+************************************************************************/
 
 char *process_variables(char *pstr, htable *pht) {
 	bool	 bs = false;
 	char	 buf[OPT_VALUE_LEN],
-		 var[OPT_NAME_LEN],
-		*pvar,
-		*pbuf;
+			 var[OPT_NAME_LEN],
+			*pvar,
+			*pbuf;
 	size_t	 size;
 
 	size = sizeof(buf);
@@ -422,7 +527,7 @@ char *process_variables(char *pstr, htable *pht) {
 					pstr++;
 					/* skip '{' */
 					pstr++;
-					pstr = parse_identifier(pstr, var, sizeof(var));
+					pstr = parse_idtf(pstr, var, sizeof(var));
 					if (pstr == NULL) {
 						/* debugf("parse_idtf returned null."); */
 						return(NULL);
@@ -482,25 +587,31 @@ char *process_variables(char *pstr, htable *pht) {
 	return(strdup(buf));
 }
 
-/*
+
+/****************
+ parse_pc_file()
+
+ DESCR
 	parse pc file
 
+ IN
 	pcfile : file to parse
 
-	return : pkgcell structure or NULL
-*/
+ OUT
+	pkgcell structure or NULL
+************************************************************************/
 
 pkgcell *parse_pc_file(char *pcfile) {
 	FILE	*fp;
 	char	*pstr,
-		*pps,
-		 buf[TMP_BUF_LEN],
-		 line[TMP_BUF_LEN];
+			*pps,
+			 buf[TMP_BUF_LEN],
+			 line[TMP_BUF_LEN];
 	pkgcell	*ppc;
 
 	fp = fopen(pcfile, "r");
 	if (fp == NULL) {
-		errorf("cannot open '%s' : %s.", pcfile, strerror(errno));
+		errorf("cannot open .pc file '%s' : %s.", pcfile, strerror(errno));
 		return(NULL);
 	}
 
@@ -513,7 +624,7 @@ pkgcell *parse_pc_file(char *pcfile) {
 	/* main parsing */
 	while (get_line(fp, line, sizeof(line)) == true) {
 		/* collect identifier */
-		pstr = parse_identifier(line, buf, sizeof(buf));
+		pstr = parse_idtf(line, buf, sizeof(buf));
 
 		switch (*pstr) {
 			case ':' :
@@ -524,7 +635,7 @@ pkgcell *parse_pc_file(char *pcfile) {
 				/* process variables */
 				pps = process_variables(pstr, ppc->variables);
 #ifdef PKGCFG_DEBUG
-debugf("keyword = '%s', value = '%s', string = '%s'", buf, pps, pstr);
+				debugf("keyword = '%s', value = '%s', string = '%s'", buf, pps, pstr);
 #endif
 				/* set variable with parsed data */
 				if (parse_keyword(ppc, buf, pps) == false) {
@@ -542,7 +653,7 @@ debugf("keyword = '%s', value = '%s', string = '%s'", buf, pps, pstr);
 				/* process variables */
 				pps = process_variables(pstr, ppc->variables);
 #ifdef PKGCFG_DEBUG
-debugf("variable = '%s', value = '%s', string = '%s'", buf, pps, pstr);
+				debugf("variable = '%s', value = '%s', string = '%s'", buf, pps, pstr);
 #endif
 				/* store variable in hash */
 				if (hash_update_dup(ppc->variables, buf, pps) == HASH_ADD_FAIL) {
@@ -556,7 +667,7 @@ debugf("variable = '%s', value = '%s', string = '%s'", buf, pps, pstr);
 			default :
 				/* unknown or empty line */
 #ifdef PKGCFG_DEBUG
-debugf("unknown = '%s'", line);
+				debugf("unknown = '%s'", line);
 #endif
 				break;
 		}
@@ -567,21 +678,27 @@ debugf("unknown = '%s'", line);
 	return(ppc);
 }
 
-/*
+
+/***************
+ pkg_cell_add()
+
+ DESCR
 	insert a package cell and return a pointer on it
 
+ IN
 	mod : module name to process
 	ppd : packages data structure
 
-	return : pkgcell structure or NULL
-*/
+ OUT
+	pkgcell structure or NULL
+************************************************************************/
 
 pkgcell *pkg_cell_add(pkgdata *ppd, char *mod) {
 	char		*pcf;
 	pkgcell		*ppc;
 
 	/* get pc file name */
-	pcf = hash_get(ppd->files, mod);
+	pcf = hash_get(ppd->files, mod); /* XXX check if NULL ?? */
 
 	/* parse pc file */
 	ppc = parse_pc_file(pcf);
@@ -593,7 +710,7 @@ pkgcell *pkg_cell_add(pkgdata *ppd, char *mod) {
 		return(NULL);
 #ifdef PKGCFG_DEBUG
 	} else {
-debugf("adding pkgcell for '%s'", mod);
+		debugf("adding pkgcell for '%s'", mod);
 #endif
 	}
 
@@ -602,7 +719,7 @@ debugf("adding pkgcell for '%s'", mod);
 
 	if (ppc->requires != NULL) {
 #ifdef PKGCFG_DEBUG
-debugf("pkgcell requires = '%s'", ppc->requires);
+		debugf("pkgcell requires = '%s'", ppc->requires);
 #endif
 		if (pkg_recurse(ppd, ppc->requires) == false) {
 			pkgcell_destroy(ppc);
@@ -613,18 +730,24 @@ debugf("pkgcell requires = '%s'", ppc->requires);
 	return(ppc);
 }
 
-/*
+
+/**************
+ pkg_recurse()
+
+ DESCR
 	recurse packages
 
+ IN
 	ppd : packages data structure
 	reqs : requires string
 
-	return : boolean
-*/
+ OUT
+	boolean
+************************************************************************/
 
 bool pkg_recurse(pkgdata *ppd, char *reqs) {
-	char		*mod;
-	dynary		*pda;
+	char			*mod;
+	dynary			*pda;
 	unsigned int	 i;
 
 #ifdef PKGCFG_DEBUG
@@ -656,22 +779,28 @@ debugf("recursing '%s'", reqs);
 	return(true);
 }
 
-/*
+
+/********************
+ pkg_single_append()
+
+ DESCR
 	append module name if it does not already exists
 
+ IN
 	ostr : modules list string
 	astr : module to append
 
+ OUT
 	return : new list of modules or NULL
-*/
+************************************************************************/
 
 char *pkg_single_append(char *ostr, char *astr) {
 	char	*pstr,
-		*buf;
+			*buf;
 	size_t	 s;
 
 #ifdef PKGCFG_DEBUG
-debugf("single_append '%s', '%s'", ostr, astr);
+	debugf("single_append '%s', '%s'", ostr, astr);
 #endif
 
 	if (*astr == CHAR_EOS)
@@ -707,35 +836,47 @@ debugf("single_append '%s', '%s'", ostr, astr);
 	return(buf);
 }
 
-/*
+
+/*****************
+ pkg_get_cflags()
+
+ DESCR
 	get recursed list of cflags
 
+ IN
 	ppd : packages data structure
 
-	return : cflags string
-*/
+ OUT
+	cflags string
+************************************************************************/
 
 char *pkg_get_cflags(pkgdata *ppd) {
 	return(pkg_get_cflags_adv(ppd, PKGCFG_CFLAGS_ALL));
 }
 
-/*
+
+/*********************
+ pkg_get_cflags_adv()
+
+ DESCR
 	get recursed list of cflags with output filtering
 
+ IN
 	ppd : packages data structure
 	opts : output filtering options
 
-	return : cflags string
-*/
+ OUT
+	cflags string
+************************************************************************/
 
 char *pkg_get_cflags_adv(pkgdata *ppd, unsigned int opts) {
-	char		*cflags = "",
-			*pstr;
-	dynary		*pda;
-	pkgcell		*ppc;
+	char			*cflags = "",
+					*pstr;
+	dynary			*pda;
+	pkgcell			*ppc;
 	unsigned int	 i,
-			 j,
-			 o;
+					 j,
+					 o;
 
 	pda = ppd->mods;
 
@@ -766,35 +907,47 @@ char *pkg_get_cflags_adv(pkgdata *ppd, unsigned int opts) {
 	return(cflags);
 }
 
-/*
+
+/***************
+ pkg_get_libs()
+
+ DESCR
 	get recursed list of libs
 
+ IN
 	ppd : packages data structure
 
-	return : libs string
-*/
+ OUT
+	libs string
+************************************************************************/
 
 char *pkg_get_libs(pkgdata *ppd) {
 	return(pkg_get_libs_adv(ppd, PKGCFG_LIBS_ALL));
 }
 
-/*
+
+/*******************
+ pkg_get_libs_adv()
+
+ DESCR
 	get recursed list of libs with output filtering
 
+ IN
 	ppd : packages data structure
 	opts : output filtering options
 
-	return : libs string
-*/
+ OUT
+	libs string
+************************************************************************/
 
 char *pkg_get_libs_adv(pkgdata *ppd, unsigned int opts) {
-	char		*libs = "",
-			*pstr;
-	dynary		*pda;
-	pkgcell		*ppc;
+	char			*libs = "",
+					*pstr;
+	dynary			*pda;
+	pkgcell			*ppc;
 	unsigned int	 i,
-			 j,
-			 o;
+					 j,
+					 o;
 
 	pda = ppd->mods;
 
@@ -839,13 +992,19 @@ char *pkg_get_libs_adv(pkgdata *ppd, unsigned int opts) {
 	return(libs);
 }
 
-/*
+
+/*****************
+ pkg_mod_exists()
+
+ DESCR
 	check if the given module exists
 
+ IN
 	mod : module to check
 
-	return : boolean
-*/
+ OUT
+	boolean
+************************************************************************/
 
 bool pkg_mod_exists(pkgdata *ppd, char *mod) {
 	if (hash_get(ppd->files, mod) == NULL) {
@@ -855,17 +1014,22 @@ bool pkg_mod_exists(pkgdata *ppd, char *mod) {
 	}
 }
 
-/*
+
+/******************
+ compare_version()
+
+ DESCR
 	compare two version strings
 
+ IN
 	vref : reference version
 	vcomp : version to check
 
-	return :
-		<0 if vref is greater than vcomp
-		=0 if vref is equal to vcomp
-		>0 if vref is smaller than vcomp
-*/
+ OUT
+	<0 if vref is greater than vcomp
+	=0 if vref is equal to vcomp
+	>0 if vref is smaller than vcomp
+************************************************************************/
 
 int compare_version(char *vref, char *vcomp) {
 	bool		 bexit = false;
@@ -938,4 +1102,3 @@ int compare_version(char *vref, char *vcomp) {
 
 	return(delta);
 }
-
