@@ -1,7 +1,7 @@
-/* $Id: premake.h,v 1.119.2.1 2005/04/25 20:49:19 mipsator Exp $ */
+/* $Id: premake.h 1974 2007-04-15 15:17:24Z coudercd $ */
 
 /*
- * Copyright (c) 2003-2004 Damien Couderc
+* Copyright (c) 2003-2006 Damien Couderc
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,30 +41,6 @@
 #include "errmsg.h"
 #include "pmk_obj.h"
 
-/* We include sys/param.h to get MAXPATHLEN.
-   This is known to work under following operating systems:
-	OpenBSD
-	FreeBSD
-	NetBSD
-	Mac OS X
-	Solaris
-	SunOS
-	HP-UX
-	AIX
-	IRIX
-	OSF1
-	Ultrix			(not verified)
-	Linux based systems	(Debian, Mandrake, RedHat, Slackware, Suse)
-	DG-UX			(not verified)
-	4.4BSD			(not verified)
-
-   Some systems does not provide the same location :
-   	Chorus			arpa/ftp.h
-
-
-   Comments about this stuff is welcome. If your system is not
-   supported then get in touch with us to fix it.
-*/
 #include <sys/param.h>
 #ifndef MAXPATHLEN
 #	define MAXPATHLEN 512
@@ -94,25 +70,25 @@
 /* major version */
 #define PREMAKE_VMAJOR	"0"
 /* minor version */
-#define PREMAKE_VMINOR	"9"
+#define PREMAKE_VMINOR	"10"
 /* minor subversion, only used for bugfixes  */
-#define PREMAKE_VSUB	"3"
+#define PREMAKE_VSUB	"2"
 
 /* full version */
-#ifndef PREMAKE_VSUB	
+#ifndef PREMAKE_VSUB
 #define PREMAKE_VFULL	PREMAKE_VMAJOR "." PREMAKE_VMINOR
 #else
 #define PREMAKE_VFULL	PREMAKE_VMAJOR "." PREMAKE_VMINOR "." PREMAKE_VSUB
 #endif
 
 /* only used for snapshots, comment for release */
-/* #define PREMAKE_SNAP		"0"		*/
+/*#define PREMAKE_SNAP		"20061028"*/
 
 /* build version string */
 #ifndef PREMAKE_SNAP
 #define PREMAKE_VERSION		PREMAKE_VFULL
 #else
-#define PREMAKE_VERSION		PREMAKE_VFULL "-s" PREMAKE_SNAP
+#define PREMAKE_VERSION		"SNAP" PREMAKE_SNAP
 #endif
 
 #define PREMAKE_CONFIG		"pmk.conf"
@@ -129,29 +105,31 @@
 */
 
 /* key names (see also pmksetup.h) */
-#define PMKCONF_BIN_AR		"BIN_AR"
-#define PMKCONF_BIN_AS		"BIN_AS"
-#define PMKCONF_BIN_AWK		"BIN_AWK"
-#define PMKCONF_BIN_CAT		"BIN_CAT"
-#define PMKCONF_BIN_CC		"BIN_CC"
-#define PMKCONF_BIN_CPP		"BIN_CPP"
-#define PMKCONF_BIN_CXX		"BIN_CXX"
-#define PMKCONF_BIN_ECHO	"BIN_ECHO"
-#define PMKCONF_BIN_EGREP	"BIN_EGREP"
-#define PMKCONF_BIN_GREP	"BIN_GREP"
-#define PMKCONF_BIN_ID		"BIN_ID"
-#define PMKCONF_BIN_INSTALL	"BIN_INSTALL"
-#define PMKCONF_BIN_LEX		"BIN_LEX"
-#define PMKCONF_BIN_LN		"BIN_LN"
+#define PMKCONF_BIN_AR			"BIN_AR"
+#define PMKCONF_BIN_AS			"BIN_AS"
+#define PMKCONF_BIN_AWK			"BIN_AWK"
+#define PMKCONF_BIN_CAT			"BIN_CAT"
+#define PMKCONF_BIN_C89			"BIN_C89"
+#define PMKCONF_BIN_C99			"BIN_C99"
+#define PMKCONF_BIN_CC			"BIN_CC"
+#define PMKCONF_BIN_CPP			"BIN_CPP"
+#define PMKCONF_BIN_CXX			"BIN_CXX"
+#define PMKCONF_BIN_ECHO		"BIN_ECHO"
+#define PMKCONF_BIN_EGREP		"BIN_EGREP"
+#define PMKCONF_BIN_GREP		"BIN_GREP"
+#define PMKCONF_BIN_ID			"BIN_ID"
+#define PMKCONF_BIN_INSTALL		"BIN_INSTALL"
+#define PMKCONF_BIN_LEX			"BIN_LEX"
+#define PMKCONF_BIN_LN			"BIN_LN"
 #define PMKCONF_BIN_PKGCONFIG	"BIN_PKGCONFIG"
-#define PMKCONF_BIN_RANLIB	"BIN_RANLIB"
-#define PMKCONF_BIN_SH		"BIN_SH"
-#define PMKCONF_BIN_STRIP	"BIN_STRIP"
-#define PMKCONF_BIN_SUDO	"BIN_SUDO"
-#define PMKCONF_BIN_TAR		"BIN_TAR"
-#define PMKCONF_BIN_YACC	"BIN_YACC"
+#define PMKCONF_BIN_RANLIB		"BIN_RANLIB"
+#define PMKCONF_BIN_SH			"BIN_SH"
+#define PMKCONF_BIN_STRIP		"BIN_STRIP"
+#define PMKCONF_BIN_SUDO		"BIN_SUDO"
+#define PMKCONF_BIN_TAR			"BIN_TAR"
+#define PMKCONF_BIN_YACC		"BIN_YACC"
 
-#define PMKCONF_HW_CPU_ARCH	"HW_CPU_ARCH"
+#define PMKCONF_HW_CPU_ARCH		"HW_CPU_ARCH"
 #define PMKCONF_HW_BYTEORDER	"HW_BYTEORDER"
 
 #define PMKCONF_PATH_BIN	"PATH_BIN"
@@ -165,7 +143,7 @@
 #define PMKCONF_OS_ARCH		"OS_ARCH"
 
 #define PMKCONF_MISC_SYSCONFDIR	"SYSCONFDIR"
-#define PMKCONF_MISC_PREFIX	"PREFIX"
+#define PMKCONF_MISC_PREFIX		"PREFIX"
 
 #define PMKCONF_AC_ECHO_N	"AC_ECHO_N"
 #define PMKCONF_AC_ECHO_C	"AC_ECHO_C"
@@ -203,12 +181,18 @@
 #define HW_ENDIAN_UNKNOWN	"UNKNOWN_ENDIAN"
 
 /* make specific variable names */
-#define MK_VAR_CFLAGS	"CFLAGS"
-#define MK_VAR_CXXFLAGS	"CXXFLAGS"
-#define MK_VAR_CPPFLAGS	"CPPFLAGS"
-#define MK_VAR_LDFLAGS	"LDFLAGS"
-#define MK_VAR_LIBS	"LIBS"
-#define MK_VAR_DEBUG	"DEBUG"
+#define MK_VAR_CFLAGS		"CFLAGS"
+#define MK_VAR_CXXFLAGS		"CXXFLAGS"
+#define MK_VAR_CPPFLAGS		"CPPFLAGS"
+#define MK_VAR_LDFLAGS		"LDFLAGS"
+#define MK_VAR_CLDFLAGS		"CLDFLAGS"
+#define MK_VAR_CXXLDFLAGS	"CXXLDFLAGS"
+#define MK_VAR_LIBS			"LIBS"
+#define MK_VAR_DEBUG		"DEBUG"
+#define MK_VAR_SL_BUILD		"SHARED_LIB_TARGETS"
+#define MK_VAR_SL_CLEAN		"SHLIB_CLEAN_TARGETS"
+#define MK_VAR_SL_INST		"SHLIB_INST_TARGETS"
+#define MK_VAR_SL_DEINST	"SHLIB_DEINST_TARGETS"
 
 /* maximal size of an error message */
 #define MAX_ERR_MSG_LEN		256
@@ -235,5 +219,6 @@
 /* string delimiter PATH variables */
 #define PATH_STR_DELIMITER	':'
 
-
 #endif /* _PREMAKE_H_ */
+
+/* vim: set noexpandtab tabstop=4 softtabstop=4 shiftwidth=4: */
