@@ -1,4 +1,4 @@
-/* $Id: comp_test.c,v 1.7 2004/11/26 22:42:40 mipsator Exp $ */
+/* $Id: comp_test.c 1442 2005-08-02 20:36:21Z mipsator $ */
 
 /* Public Domain */
 
@@ -9,13 +9,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "../common.h"
-#include "../compat/compat.h"
 #include "../detect.h"
-#include "../dynarray.h"
-#include "../hash.h"
-#include "../parse.h"
-#include "../pmk_obj.h"
 
 int main(int argc, char *argv[]) {
 	char		*cc;
